/*
 * JListaElementos.java
 *
 * Created on 13 de enero de 2005, 17:05
 */

package utiles;

/**
 *Objeto estandar Lista de elementos
 */
public class JListaElementos implements IListaElementos , Cloneable {
    private static final long serialVersionUID = 33333326L;

    public static final int mclIncrementoDefecto = 16;

    private Object[] moListDatos = new Object[0];
    private int mlSize = 0;
    private int mlIncremento = mclIncrementoDefecto;
    private static final int MAX_ARRAY_SIZE = Integer.MAX_VALUE - 8;
    
    /**
     * Constructor 
     */
    public JListaElementos() {
        setIncrementoTamano(mclIncrementoDefecto);
    }
    /**
     * Constructor 
     * @param plIncremento incremento minimo del array de memoria, 
     *                     poner mas alto cuando haya muchas inserciones
     */
    public JListaElementos(int plIncremento) {
        setIncrementoTamano(plIncremento);
    }
    /**
     * establece el incremento de tamano de los array
     * @param plIncremento incremento del array 
     */
    public void setIncrementoTamano(int plIncremento){
        if(plIncremento>0){
            mlIncremento =plIncremento;
        }
    }
    public int getIncremento(){
        return mlIncremento;
    }
    private void comprobarYRedimensionarMas1(int minCapacity) {
        // overflow-conscious code
        int oldCapacity = moListDatos.length;
        int newCapacity = oldCapacity << 1;
        if (newCapacity - minCapacity < 0) {
            newCapacity = minCapacity;
        }
        if (newCapacity - MAX_ARRAY_SIZE > 0) {
            newCapacity = hugeCapacity(minCapacity);
        }
        // minCapacity is usually close to size, so this is a win:
        Object[] loListDatos = moListDatos;
        moListDatos = new Object[newCapacity];
        System.arraycopy(loListDatos, 0, moListDatos, 0, mlSize);
    }   
    private static int hugeCapacity(int minCapacity) {
        if (minCapacity < 0) {
            throw new OutOfMemoryError();
        }
        return (minCapacity > MAX_ARRAY_SIZE) ?
            Integer.MAX_VALUE :
            MAX_ARRAY_SIZE;
    }    
    private void comprobarYRedimensionarMas1() {
        if((mlSize+1) > moListDatos.length ){
            if(mclIncrementoDefecto==mlIncremento && moListDatos.length!=0){
                comprobarYRedimensionarMas1(mlSize+1);
            }else{
                Object[] loListDatos = moListDatos;
                moListDatos = new Object[moListDatos.length + mlIncremento];
                System.arraycopy(loListDatos, 0, moListDatos, 0, mlSize);
            }
        }
    }

    public boolean add(Object poFilaDatos){
        comprobarYRedimensionarMas1();
        moListDatos[mlSize] = poFilaDatos;
        mlSize++;
        return true;
    }

    public int size(){
        return mlSize;
    }
    public void insertElementAt(Object obj, int index) {
	if (index > mlSize) {
	    throw new ArrayIndexOutOfBoundsException(index+ " > " + mlSize);
	}
	comprobarYRedimensionarMas1();
	System.arraycopy(moListDatos, index, moListDatos, index + 1, mlSize - index);
	moListDatos[index] = obj;
	mlSize++;
    }
    public Object get(int i){
//        if((i>=mlSize)||(i<0)){
//            throw new ArrayIndexOutOfBoundsException();
//        }
        return moListDatos[i];
    }

    public Object remove(int index){
//        if((index>=mlSize)||(index<0)){
//            throw new ArrayIndexOutOfBoundsException();
//        }
        Object loObj = moListDatos[index];

	int lAMover = mlSize - index - 1;
	if (lAMover > 0){
	    System.arraycopy(moListDatos, index+1, moListDatos, index,
			     lAMover);
        }
	mlSize--;
        moListDatos[mlSize] = null; // Let gc do its work

        return loObj;

    }

    public int indexOf(Object poObject){
//        Object loObj = null;
        for(int i =0; i<mlSize;i++){
            if(moListDatos[i] == poObject){
                return i;
            }
        }
        return -1;
    }

    public Object remove(Object poObject){
        Object loObj = null;
        for(int i =0; i<mlSize;i++){
            if(moListDatos[i] == poObject){
                loObj = remove(i);
            }
        }
        return loObj;
    }

    public void clear(){
        for(int i = 0 ; i < mlSize; i++){
            moListDatos[i] = null;
        }
        mlSize = 0;
    }
    public IListaElementosIterator iterator(){
        return new JListaElementosIterator(this);
    }
    public Object[] toArray(){
       Object[] loList = new Object[mlSize];
       System.arraycopy(moListDatos, 0, loList, 0, mlSize);
       return loList;
    }
    public String[] toArrayString(){
       String[] loList = new String[mlSize];
       for(int i = 0 ; i < loList.length; i++){
           if(moListDatos[i]!=null){
            loList[i]=moListDatos[i].toString();
           }
       }
       System.arraycopy(moListDatos, 0, loList, 0, mlSize);
       return loList;
    }
    public Object clone() throws CloneNotSupportedException{
        JListaElementos loLista = (JListaElementos) super.clone();
       Object[] loList = new Object[mlSize];
       System.arraycopy(moListDatos, 0, loList, 0, mlSize);
       loLista.moListDatos=loList;

        return loLista;
    }
}
