/*
 * ConnectionWrapper.java
 *
 * Created on 2 de agosto de 2003, 1:40
 */


package utilesBD.poolConexiones;

import java.sql.*;
import java.util.Properties;
/**encapsula una conexion, pero en vez de cerrar la conexion la devuelve a la lista de conexiones*/
public class ConnectionWrapper implements Connection {
    private Connection moConexion;
    private final IPoolObjetos moPool ;
    /**
     * Creates a new instance of ConnectionWrapper
     * @param poPool pool de cnex.
     * @param poConexion conexion concreta
     */
    public ConnectionWrapper(IPoolObjetos poPool ,Connection poConexion) {
        moConexion = poConexion;
        moPool = poPool;
    }

    /**
     * borrar warnig
     * @throws SQLException error
     */
    public void clearWarnings() throws SQLException {
        moConexion.clearWarnings();
    }

    /**Devuelve la conexion a la lista de conexiones*/
    public void close() {
        if (moConexion != null){
            moPool.returnConnection(moConexion);
        }
        moConexion = null;
    }
    /**finalizar close*/
    protected void finalize() throws Throwable {
        close();
        super.finalize();
    }
    /**
     * realiza el commit
     * @throws SQLException error
     */
    public void commit() throws SQLException {
        moConexion.commit();
    }
    /**
     * crea un stament
     * @return Statement
     * @throws SQLException error
     */
    public Statement createStatement() throws SQLException {
        return moConexion.createStatement();
    }
    
    
    /**
     * Devuelve si es autocommit
     * @throws SQLException error
     * @return  si es autocommit
     */
    public boolean getAutoCommit() throws SQLException {
        return moConexion.getAutoCommit();
    }
    /**
     * devuelve el catalogo
     * @throws SQLException erro
     * @return  el catalogo
     */
    public String getCatalog() throws SQLException {
        return moConexion.getCatalog();
    }
    
    /**
     * devuelve los metadatos de la base de datos
     * @throws SQLException error
     * @return DatabaseMetaData 
     */
    public DatabaseMetaData getMetaData() throws SQLException {
        return moConexion.getMetaData();
    }
    /**
     * devuelve el nivel de transaccion
     * @throws SQLException error
     * @return nivel 
     */
    public int getTransactionIsolation() throws SQLException {
        return moConexion.getTransactionIsolation();
    }
    
    /**
     * Deveulve los warning
     * @throws SQLException error 
     * @return SQLWarning
     */
    public SQLWarning getWarnings() throws SQLException {
        return moConexion.getWarnings();
    }
    /**
     * devuelve si la conexion es cerrada
     * @throws SQLException error
     * @return si es cerrada
     */
    public boolean isClosed() throws SQLException {
        return moConexion.isClosed();
    }
    /**
     * devuelve si la conexion es solo lectura
     * @throws SQLException error
     * @return si es de solo lectura
     */
    public boolean isReadOnly() throws SQLException {
        return moConexion.isReadOnly();
    }
    /**
     * devuelve sql nativo
     * @param sql sql
     * @throws SQLException error
     * @return sql nativo
     */
    public String nativeSQL(String sql) throws SQLException {
        return moConexion.nativeSQL(sql);
    }
    /**
     * Devuelve un stament preparado en funcion de la SQL pasada por parametro
     * @param sql sql
     * @throws SQLException error
     * @return CallableStatement
     */
    public CallableStatement prepareCall(String sql) throws SQLException {
        return moConexion.prepareCall(sql);
    }
    
    /**compila la sql */
    public PreparedStatement prepareStatement(String sql) throws SQLException {
        return moConexion.prepareStatement(sql);
    }
    
    /**
     * roolback
     * @throws SQLException error
     */
    public void rollback() throws SQLException {
        moConexion.rollback();
    }
    
//    
    /**
     * establecemos si es auto commit
     * @param autoCommit valor
     * @throws SQLException error
     */
    public void setAutoCommit(boolean autoCommit) throws SQLException {
        moConexion.setAutoCommit(autoCommit);
    }
    
    /**
     * establecemos el catalogo
     * @param catalog catalogo
     * @throws SQLException error
     */
    public void setCatalog(String catalog) throws SQLException {
        moConexion.setCatalog(catalog);
    }
    
    /**
     * establecemos si es de solo lectura
     * @param readOnly valor
     * @throws SQLException error
     */
    public void setReadOnly(boolean readOnly) throws SQLException {
        moConexion.setReadOnly(readOnly);
    }
    
    
    /**
     * establecemos el nivel de transaccion
     * @param level nivel
     * @throws SQLException error
     */
    public void setTransactionIsolation(int level) throws SQLException {
        moConexion.setTransactionIsolation(level); 
    }
    
    
    /////////////////////////////////77
    ////////// version JVM 1.2
    /////////////////////////////////77
    
           
    public java.util.Map getTypeMap() throws SQLException {
        return moConexion.getTypeMap();
    }
    public void setTypeMap(java.util.Map map) throws SQLException {
        moConexion.setTypeMap(map);
    }       
    public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
        return moConexion.prepareCall(sql, resultSetType, resultSetConcurrency);
    }
    public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
        return moConexion.prepareStatement(sql, resultSetType, resultSetConcurrency);
    }
    public Statement createStatement(int resultSetType, int resultSetConcurrency) throws SQLException {
        return moConexion.createStatement(resultSetType, resultSetConcurrency);
    }
        
    /////////////////////////////////77
    ////////// version JVM 1.4
    /////////////////////////////////77
            
    
    
    
    
    
    
    
    
    
    
    
    
    

    public Savepoint setSavepoint() throws SQLException {
        return moConexion.setSavepoint();
    }
    
    public Savepoint setSavepoint(String name) throws SQLException {
        return moConexion.setSavepoint(name);
    }
    
    public void setHoldability(int holdability) throws SQLException {
        moConexion.setHoldability(holdability);
    }
    
    public Statement createStatement(int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
        return moConexion.createStatement(resultSetType, resultSetConcurrency, resultSetHoldability);
    }

    
    public PreparedStatement prepareStatement(String sql, String[] columnNames) throws SQLException {
        return moConexion.prepareStatement(sql, columnNames);
    }
    
    public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys) throws SQLException {
        return moConexion.prepareStatement(sql, autoGeneratedKeys);
    }
    
    public PreparedStatement prepareStatement(String sql, int[] columnIndexes) throws SQLException {
        return moConexion.prepareStatement(sql, columnIndexes);
    }
    

    public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
        return moConexion.prepareStatement(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
    }
    
    public void releaseSavepoint(Savepoint savepoint) throws SQLException {
        moConexion.releaseSavepoint(savepoint);
    }

    public int getHoldability() throws SQLException {
        return moConexion.getHoldability();
    }
    
    public void rollback(Savepoint savepoint) throws SQLException {
        moConexion.rollback(savepoint);
    }
    
    public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
        return moConexion.prepareCall(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
    }



    
    /////////////////////////////////77
    ////////// version JVM 1.6
    /////////////////////////////////77
    
    public Clob createClob() throws SQLException {
        return moConexion.createClob();
    }

    public Blob createBlob() throws SQLException {
        return moConexion.createBlob();
    }

    public NClob createNClob() throws SQLException {
        return moConexion.createNClob();
    }

    public SQLXML createSQLXML() throws SQLException {
        return moConexion.createSQLXML();
    }

    public boolean isValid(int timeout) throws SQLException {
        return moConexion.isValid(timeout);
    }

    public void setClientInfo(String name, String value) throws SQLClientInfoException {
        moConexion.setClientInfo(name, value);
    }

    public void setClientInfo(Properties properties) throws SQLClientInfoException {
        moConexion.setClientInfo(properties);
    }

    public String getClientInfo(String name) throws SQLException {
        return moConexion.getClientInfo(name);
    }

    public Properties getClientInfo() throws SQLException {
        return moConexion.getClientInfo();
    }

    public Array createArrayOf(String typeName, Object[] elements) throws SQLException {
        return moConexion.createArrayOf(typeName, elements);
    }

    public Struct createStruct(String typeName, Object[] attributes) throws SQLException {
        return moConexion.createStruct(typeName, attributes);
    }

    public Object unwrap(Class iface) throws SQLException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public boolean isWrapperFor(Class iface) throws SQLException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public int getNetworkTimeout() throws SQLException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void setNetworkTimeout(java.util.concurrent.Executor executor, int milliseconds) throws SQLException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void setSchema(String schema) throws SQLException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public String getSchema() throws SQLException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void abort(java.util.concurrent.Executor executor) throws SQLException {
        throw new UnsupportedOperationException("Not supported yet.");
    }


    
}
