/*
 * PoolObjetos.java
 *
 * Created on 30 de julio de 2003, 13:27
 */
package utilesBD.poolConexiones;



import ListDatos.*;
import java.sql.*;
import utiles.*;
import utilesBD.estructuraBD.JConstructorEstructuraBDConnection;


/**pool de conexiones*/
public class PoolObjetosImpl implements IPoolObjetos {
    private long mlTiempoExpiracion=4*60*1000;

    private final JListaElementos bloqueados = new JListaElementos();
    private final JListaElementos nobloqueados = new JListaElementos();
    
    private int mlNumeroConexiones=10;

    private JConstructorEstructuraBDConnection moEstructura;
    
    private String msConexion = null;
    private String msClass = null;
    private String msUsuario = null;
    private String msPassword = null;
    private String msTipoSQL = null;
    private boolean mbCerrarConexionDespuesUsar=false;
    private boolean mbUsarSiguienteConexionNueva=false;
    
    public PoolObjetosImpl(int plNumeroConexiones,
        String psConexion, String psClass, String psUsuario, String psPassword, String psTipoSQL
            ) {
        mlNumeroConexiones=plNumeroConexiones;
        msConexion = psConexion;
        msClass = psClass;
        msUsuario = psUsuario ;
        msPassword = psPassword;
        msTipoSQL=psTipoSQL;
    }
    /**Despues de cada operacion cierra la conexion*/
    public synchronized void setCerrarConexionDespuesUsar(boolean pbCerrar){
        mbCerrarConexionDespuesUsar=pbCerrar;
    }
    /**Obliga a que la siguiente conexion sea nueva*/
    public synchronized void setUsarSiguienteConexionNueva(boolean pbNueva){
        mbUsarSiguienteConexionNueva=pbNueva;
    }
    /**Devuelve una conexion*/
    private synchronized JElementoConnection checkOut() throws SQLException{
        long ahora = System.currentTimeMillis();
        while(mlBloqueados()>=mlNumeroConexiones){
            JDepuracion.anadirTexto(0,PoolObjetosImpl.class.getName(),"PoolObjetos:Antes de bloquear conexion");
            try{
                wait(5 * 1000);
            }catch(Exception e){
                //vacio
            }
//DIABLOS!!! puede ser q se cierre en mitad de una operacion, ASI QUE DOBLAMOS EL TIEMPO EXPIRACION
            //NO USAR ITERADORES
            Object[] lao = bloqueados.toArray();
            for (int i = 0 ; i < lao.length; i++){
                JElementoConnection loElem = (JElementoConnection) lao[i];
                ahora = System.currentTimeMillis();
                if ( ( ahora - loElem.mlAhora ) > mlTiempoExpiracion*2){
                    //el objeto a expirado
                    bloqueados.remove(loElem);
                    expirar( loElem );loElem = null;
                }
            }
            JDepuracion.anadirTexto(JDepuracion.mclINFORMACION,PoolObjetosImpl.class.getName(),"PoolObjetos:Despues de Bloquear conexion");
        }
        if( nobloqueados.size()>0){
            //1 expiramos todas las conexiones q tengan q expirar
            limpiarExpirados();
            //2 vemos conexion activa
            Object[] lao = nobloqueados.toArray();
            if(lao.length>0){
                //si la siguiente conexion nueva
                if(mbUsarSiguienteConexionNueva){
                    //si ha alcanzado el numero de conexiones maximas cierra una
                    if(lao.length>=mlNumeroConexiones){
                        JElementoConnection loElem = (JElementoConnection) lao[0];
                        nobloqueados.remove (loElem);
                        expirar(loElem);
                    }
                    mbUsarSiguienteConexionNueva=false;
                }else{
                    //devuelve una conexion y la mete en la lista de bloqueados
                    JElementoConnection loElem = (JElementoConnection) lao[0];
                    nobloqueados.remove (loElem);
                    bloqueados.add(loElem);
                    return loElem;
                }
            }
        }
        JElementoConnection loElem = crear();
        bloqueados.add(loElem);
        JDepuracion.anadirTexto(JDepuracion.mclINFORMACION,PoolObjetosImpl.class.getName(),"PoolObjetos bloqueados " +String.valueOf(mlBloqueados()));
        JDepuracion.anadirTexto(JDepuracion.mclINFORMACION,PoolObjetosImpl.class.getName(),"PoolObjetos NO bloqueados " +String.valueOf(mlNOBloqueados()));
        return loElem ;
    }
    public synchronized void limpiarExpirados() {
        long ahora = System.currentTimeMillis();
        JElementoConnection o;
        //1 expiramos todas las conexiones q tengan q expirar
        Object[] lao = nobloqueados.toArray();
        for (int i = 0 ; i < lao.length; i++){
            o = (JElementoConnection) lao[i];
            if ( ( ahora - o.mlAhora ) > mlTiempoExpiracion){
                //el objeto a expirado
                nobloqueados.remove(o);
                expirar( o );o = null;
            } else {
                if (!validar(o)){
                    nobloqueados.remove(o);
                    expirar(o);
                    o=null;
                }
            }
        }
        notifyAll();
        JDepuracion.anadirTexto(JDepuracion.mclINFORMACION,PoolObjetosImpl.class.getName(),"PoolObjetos bloqueados " +String.valueOf(bloqueados.size()));
        JDepuracion.anadirTexto(JDepuracion.mclINFORMACION,PoolObjetosImpl.class.getName(),"PoolObjetos NO bloqueados " +String.valueOf(nobloqueados.size()));
    }
    private synchronized int mlBloqueados() throws SQLException {
        int lBloq = 0;
        Object[] lao = bloqueados.toArray();
        for (int i = 0 ; i < lao.length; i++){
            JElementoConnection o = (JElementoConnection) lao[i];
            if(validar(o)){
                lBloq++;
            }
        }
        return lBloq;
    }
    private synchronized int mlNOBloqueados() throws SQLException {
        int lBloq = 0;
        Object[] lao = nobloqueados.toArray();
        for (int i = 0 ; i < lao.length; i++){
            JElementoConnection o = (JElementoConnection) lao[i];
            if(validar(o)){
                lBloq++;
            }
        }
        return lBloq ;
    }
    private synchronized void checkIn(Connection poConex){
        JElementoConnection loResult = null;
        Object[] lao = bloqueados.toArray();
        for (int i = 0 ; i < lao.length; i++){
            JElementoConnection o = (JElementoConnection) lao[i];
            if(o.moConex==poConex){
                loResult = o;
            }
        }
        if(loResult!=null){
            bloqueados.remove(loResult);
            if(mbCerrarConexionDespuesUsar){
                expirar(loResult);
            }else{
                nobloqueados.add(loResult);
            }
        }else{
            expirar(new JElementoConnection(poConex, 0));
        }
        notifyAll();
        JDepuracion.anadirTexto(JDepuracion.mclINFORMACION,PoolObjetosImpl.class.getName(),"PoolObjetos bloqueados " +bloqueados.size());
        JDepuracion.anadirTexto(JDepuracion.mclINFORMACION,PoolObjetosImpl.class.getName(),"PoolObjetos NO bloqueados " +nobloqueados.size());
    }
    private synchronized JElementoConnection crear() throws SQLException{
        String lsConexion = null;
        String lsClass = null;
        String lsUsuario = null;
        String lsPassword = null;
        try{
            lsClass = msClass;
            lsConexion = msConexion;
            lsUsuario = msUsuario;
            lsPassword = msPassword;

            Class.forName(lsClass);
        }catch(Exception e){
            JDepuracion.anadirTexto(PoolObjetosImpl.class.getName(), e);
        }
        
        return new JElementoConnection(DriverManager.getConnection(lsConexion, lsUsuario , lsPassword ), System.currentTimeMillis());
    }
    private synchronized void expirar(JElementoConnection o) {
        try{
            o.moConex.close();
        }catch(Throwable e){
            JDepuracion.anadirTexto(PoolObjetosImpl.class.getName(), e);
        }
        notifyAll();
    }
    private synchronized boolean validar(JElementoConnection o) {
        try {
            return o!=null && o.moConex!=null && (!o.moConex.isClosed());
        } catch (Throwable ex) {
            JDepuracion.anadirTexto(PoolObjetosImpl.class.getName(), ex);
            return false;
        }
    }
    /**
     * Devuelve una conexion libre/creada
     * @return conexion
     * @throws SQLException error
     */
    public Connection getConnection() throws SQLException{
        return new ConnectionWrapper(this,checkOut().moConex );
    }
    /**devuelve la conexion a la lista de conexiones libres*/
    public void returnConnection(Connection c){
        checkIn(c);
    }
    /**
     * Devuelve una conexion libre/creada
     * @param username usuario
     * @param password password
     * @throws SQLException error
     * @return conexion
     */
    public Connection getConnection(String username, String password) throws SQLException{
        return new ConnectionWrapper(this, checkOut().moConex);
    }

    /**
     * Deveulve el motor de conversion de objeto a SQL
     * @return select motor
     */
    public ISelectMotor getSelectMotor() throws Exception {
        String lsTipo = msTipoSQL;
        ISelectMotor loSelect = JSelectMotorFactory.getInstance().getSelectMotor(lsTipo);
        if(loSelect==null){
            JSelectMotorFactory.getInstance().registrarMotor(lsTipo, (ISelectMotor)Class.forName(lsTipo).newInstance(), lsTipo);
            loSelect = JSelectMotorFactory.getInstance().getSelectMotor(lsTipo);
        }
        return loSelect;
    }

    public synchronized JConstructorEstructuraBDConnection getEstructura(Connection poConex) throws SQLException{
        //se hace asi pq se supone q el poolObjetos continen conexiones a la misma BD
        if(moEstructura == null){
            moEstructura = new JConstructorEstructuraBDConnection(poConex, true);
        }
        //para aplicaciones locales siempre es valida la ultima conexion
        moEstructura.setConec(poConex);
        return moEstructura;
//        return new JConstructorEstructuraBDConnection(poConex);
    }
    public JServerServidorDatos getServidorDatos() throws Exception {
        JServerServidorDatos loServer = new JServerServidorDatos();
        loServer.setTipo(JServerServidorDatos.mclTipoBD);
        loServer.setConec(getConnection());
        loServer.setSelect(getSelectMotor());
        loServer.setConstrucEstruc(getEstructura(loServer.getConec()));
        return loServer;
    }

    public void close() {
        if( nobloqueados.size()>0){
            Object[] lao = nobloqueados.toArray();
            for (int i = 0 ; i < lao.length; i++){
                expirar((JElementoConnection) lao[i]);
            }
        }
        if( bloqueados.size()>0){
            Object[] lao = bloqueados.toArray();
            for (int i = 0 ; i < lao.length; i++){
                expirar((JElementoConnection) lao[i]);
            }
        }
        nobloqueados.clear();
        bloqueados.clear();
        
    }

    /**
     * @return the mlTiempoExpiracion
     */
    public long getTiempoExpiracion() {
        return mlTiempoExpiracion;
    }

    /**
     * @param mlTiempoExpiracion the mlTiempoExpiracion to set
     */
    public void setTiempoExpiracion(long mlTiempoExpiracion) {
        this.mlTiempoExpiracion = mlTiempoExpiracion;
    }
}
class JElementoConnection {
    Connection moConex;
    long mlAhora;
    public JElementoConnection (Connection poConex, long plahora){
        moConex=poConex;
        mlAhora=plahora;
    }
}